@@ -21,6 +21,7 @@ package v1
 import (
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
 )
 
 // +k8s:openapi-gen=true
@@ -46,7 +47,6 @@ type CheClusterSpec struct {
 	// Configuration settings related to the metrics collection used by the Che installation.
 	// +optional
 	Metrics CheClusterSpecMetrics `json:"metrics"`
-
 	// Configuration settings specific to Che installations made on upstream Kubernetes.
 	// +optional
 	K8s CheClusterSpecK8SOnly `json:"k8s"`
@@ -157,6 +157,12 @@ type CheClusterSpecServer struct {
 	// Overrides the memory request used in the Devfile registry deployment. Defaults to 16Mi.
 	// +optional
 	DevfileRegistryMemoryRequest string `json:"devfileRegistryMemoryRequest,omitempty"`
+	// Devfile registry ingress custom settings
+	// +optional
+	DevfileRegistryIngress IngressCustomSettings `json:"devfileRegistryIngress,omitempty"`
+	// Devfile registry route custom settings
+	// +optional
+	DevfileRegistryRoute RouteCustomSettings `json:"devfileRegistryRoute,omitempty"`
 	// Instructs the operator on whether or not to deploy a dedicated Devfile registry server.
 	// By default a dedicated devfile registry server is started.
 	// But if `externalDevfileRegistry` is `true`, then no such dedicated server will be started by the operator
@@ -169,7 +175,7 @@ type CheClusterSpecServer struct {
 	// +optional
 	PluginRegistryUrl string `json:"pluginRegistryUrl,omitempty"`
 	// Overrides the container image used in the Plugin registry deployment. This includes the image tag.
-	// Omit it or leave it empty to use the defaut container image provided by the operator.
+	// Omit it or leave it empty to use the default container image provided by the operator.
 	// +optional
 	PluginRegistryImage string `json:"pluginRegistryImage,omitempty"`
 	// Overrides the image pull policy used in the Plugin registry deployment.
@@ -182,6 +188,12 @@ type CheClusterSpecServer struct {
 	// Overrides the memory request used in the Plugin registry deployment. Defaults to 16Mi.
 	// +optional
 	PluginRegistryMemoryRequest string `json:"pluginRegistryMemoryRequest,omitempty"`
+	// Plugin registry ingress custom settings
+	// +optional
+	PluginRegistryIngress IngressCustomSettings `json:"pluginRegistryIngress,omitempty"`
+	// Plugin registry route custom settings
+	// +optional
+	PluginRegistryRoute RouteCustomSettings `json:"pluginRegistryRoute,omitempty"`
 	// Instructs the operator on whether or not to deploy a dedicated Plugin registry server.
 	// By default a dedicated plugin registry server is started.
 	// But if `externalPluginRegistry` is `true`, then no such dedicated server will be started by the operator
@@ -208,7 +220,8 @@ type CheClusterSpecServer struct {
 	// (see also the `proxyURL` and `nonProxyHosts` fields).
 	// +optional
 	ProxyPort string `json:"proxyPort,omitempty"`
-	// List of hosts that should not use the configured proxy. Use `|`` as delimiter, eg `localhost|my.host.com|123.42.12.32`
+	// List of hosts that should not use the configured proxy.
+	// So specify wild card domain use the following form `.<DOMAIN>` and `|` as delimiter, eg: `localhost|.my.host.com|123.42.12.32`
 	// Only use when configuring a proxy is required.
 	// Operator respects OpenShift cluster wide proxy configuration and no additional configuration is required,
 	// but defining `nonProxyHosts` in a custom resource leads to merging non proxy hosts lists from the
@@ -236,6 +249,37 @@ type CheClusterSpecServer struct {
 	// Overrides the memory limit used in the Che server deployment. Defaults to 1Gi.
 	// +optional
 	ServerMemoryLimit string `json:"serverMemoryLimit,omitempty"`
+
+	// Sets the server and workspaces exposure type. Possible values are "multi-host", "single-host", "default-host".
+	// Defaults to "multi-host" which creates a separate ingress (or route on OpenShift) for every required
+	// endpoint.
+	// "single-host" makes Che exposed on a single hostname with workspaces exposed on subpaths. Please read the docs
+	// to learn about the limitations of this approach. Also consult the `singleHostExposureType` property to further configure
+	// how the operator and Che server make that happen on Kubernetes.
+	// "default-host" exposes che server on the host of the cluster. Please read the docs to learn about
+	// the limitations of this approach.
+	// +optional
+	ServerExposureStrategy string `json:"serverExposureStrategy,omitempty"`
+
+	// The image used for the gateway in the single host mode.
+	// Omit it or leave it empty to use the defaut container image provided by the operator.
+	// +optional
+	SingleHostGatewayImage string `json:"singleHostGatewayImage,omitempty"`
+
+	// The image used for the gateway sidecar that provides configuration to the gateway.
+	// Omit it or leave it empty to use the defaut container image provided by the operator.
+	// +optional
+	SingleHostGatewayConfigSidecarImage string `json:"singleHostGatewayConfigSidecarImage,omitempty"`
+
+	// The labels that need to be present (and are put) on the configmaps representing the gateway configuration.
+	// +optional
+	SingleHostGatewayConfigMapLabels labels.Set `json:"singleHostGatewayConfigMapLabels,omitempty"`
+	// Che server ingress custom settings
+	// +optional
+	CheServerIngress IngressCustomSettings `json:"cheServerIngress,omitempty"`
+	// Che server route custom settings
+	// +optional
+	CheServerRoute RouteCustomSettings `json:"cheServerRoute,omitempty"`
 }
 
 // +k8s:openapi-gen=true
@@ -366,6 +410,26 @@ type CheClusterSpecAuth struct {
 	// Default value is `Always` for `nightly` or `latest` images, and `IfNotPresent` in other cases.
 	// +optional
 	IdentityProviderImagePullPolicy corev1.PullPolicy `json:"identityProviderImagePullPolicy,omitempty"`
+	// Ingress custom settings
+	// +optional
+	IdentityProviderIngress IngressCustomSettings `json:"identityProviderIngress,omitempty"`
+	// Route custom settings
+	// +optional
+	IdentityProviderRoute RouteCustomSettings `json:"identityProviderRoute,omitempty"`
+}
+
+// Ingress custom settings, can be extended in the future
+type IngressCustomSettings struct {
+	// Comma separated list of labels that can be used to organize and categorize (scope and select) objects.
+	// +optional
+	Labels string `json:"labels,omitempty"`
+}
+
+// Route custom settings, can be extended in the future
+type RouteCustomSettings struct {
+	// Comma separated list of labels that can be used to organize and categorize (scope and select) objects.
+	// +optional
+	Labels string `json:"labels,omitempty"`
 }
 
 // +k8s:openapi-gen=true
@@ -407,6 +471,8 @@ type CheClusterSpecK8SOnly struct {
 	// Strategy for ingress creation. This can be `multi-host` (host is explicitly provided in ingress),
 	// `single-host` (host is provided, path-based rules) and `default-host.*`(no host is provided, path-based rules).
 	// Defaults to `"multi-host`
+	// Deprecated in favor of "serverExposureStrategy" in the "server" section, which defines this regardless of the cluster type.
+	// If both are defined, `serverExposureStrategy` takes precedence.
 	// +optional
 	IngressStrategy string `json:"ingressStrategy,omitempty"`
 	// Ingress class that will define the which controler will manage ingresses. Defaults to `nginx`.
@@ -423,6 +489,14 @@ type CheClusterSpecK8SOnly struct {
 	// ID of the user the Che pod and Workspace pods containers should run as. Default to `1724`.
 	// +optional
 	SecurityContextRunAsUser string `json:"securityContextRunAsUser,omitempty"`
+	// When the serverExposureStrategy is set to "single-host", the way the server, registries and workspaces
+	// are exposed is further configured by this property. The possible values are "native" (which means
+	// that the server and workspaces are exposed using ingresses on K8s) or "gateway" where the server
+	// and workspaces are exposed using a custom gateway based on Traefik. All the endpoints whether backed by the ingress
+	// or gateway "route" always point to the subpaths on the same domain.
+	// Defaults to "native".
+	// +optional
+	SingleHostExposureType string `json:"singleHostExposureType,omitempty"`
 }
 
 type CheClusterSpecMetrics struct {
