@@ -14,12 +14,14 @@ package deploy
 
 import (
 	"fmt"
+	"gopkg.in/yaml.v2"
 	"io/ioutil"
 	"os"
 	"strings"
 
-	util "github.com/eclipse/che-operator/pkg/util"
+	"github.com/eclipse/che-operator/pkg/util"
 	"github.com/sirupsen/logrus"
+	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/apps/v1"
 	"k8s.io/client-go/kubernetes/scheme"
 
@@ -27,18 +29,24 @@ import (
 )
 
 var (
-	defaultCheServerImage                string
-	defaultCheVersion                    string
-	defaultPluginRegistryImage           string
-	defaultDevfileRegistryImage          string
-	defaultCheTLSSecretsCreationJobImage string
-	defaultPvcJobsImage                  string
-	defaultPostgresImage                 string
-	defaultKeycloakImage                 string
+	defaultCheServerImage                      string
+	defaultCheVersion                          string
+	defaultPluginRegistryImage                 string
+	defaultDevfileRegistryImage                string
+	defaultCheTLSSecretsCreationJobImage       string
+	defaultPvcJobsImage                        string
+	defaultPostgresImage                       string
+	defaultKeycloakImage                       string
+	defaultSingleHostGatewayImage              string
+	defaultSingleHostGatewayConfigSidecarImage string
 
 	defaultCheWorkspacePluginBrokerMetadataImage  string
 	defaultCheWorkspacePluginBrokerArtifactsImage string
 	defaultCheServerSecureExposerJwtProxyImage    string
+	DefaultSingleHostGatewayConfigMapLabels       = map[string]string{
+		"app":       "che",
+		"component": "che-gateway-config",
+	}
 )
 
 const (
@@ -48,7 +56,6 @@ const (
 	DefaultChePostgresDb       = "dbche"
 	DefaultPvcStrategy         = "per-workspace"
 	DefaultPvcClaimSize        = "1Gi"
-	DefaultIngressStrategy     = "multi-host"
 	DefaultIngressClass        = "nginx"
 
 	DefaultPluginRegistryMemoryLimit   = "256Mi"
@@ -76,6 +83,10 @@ const (
 	DefaultSecurityContextFsGroup   = "1724"
 	DefaultSecurityContextRunAsUser = "1724"
 
+	DefaultServerExposureStrategy           = "multi-host"
+	DefaultKubernetesSingleHostExposureType = "native"
+	DefaultOpenShiftSingleHostExposureType  = "gateway"
+
 	// This is only to correctly  manage defaults during the transition
 	// from Upstream 7.0.0 GA to the next version
 	// That fixed bug https://github.com/eclipse/che/issues/13714
@@ -96,30 +107,6 @@ func InitDefaults(defaultsPath string) {
 	}
 }
 
-func InitDefaultsFromEnv() {
-	defaultCheVersion = getDefaultFromEnv("CHE_VERSION")
-	defaultCheServerImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_server"))
-	defaultPluginRegistryImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_plugin_registry"))
-	defaultDevfileRegistryImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_devfile_registry"))
-	defaultPvcJobsImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_pvc_jobs"))
-	defaultPostgresImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_postgres"))
-	defaultKeycloakImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_keycloak"))
-	fmt.Printf("DEBUG s390x :: Got util.GetArchitectureDependentEnv('RELATED_IMAGE_keycloak') = '%s'", util.GetArchitectureDependentEnv("RELATED_IMAGE_keycloak"))
-	fmt.Printf("DEBUG s390x :: Got defaultKeycloakImage = getDefaultFromEnv(util.GetArchitectureDependentEnv('RELATED_IMAGE_keycloak'))= '%s'", defaultKeycloakImage)
-
-	// CRW images for that are mentioned in the Che server che.properties
-	// For CRW these should be synced by hand with images stored in RH registries
-	// instead of being synced by script with the content of the upstream `che.properties` file
-	defaultCheWorkspacePluginBrokerMetadataImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_workspace_plugin_broker_metadata"))
-	defaultCheWorkspacePluginBrokerArtifactsImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_workspace_plugin_broker_artifacts"))
-	defaultCheServerSecureExposerJwtProxyImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_server_secure_exposer_jwt_proxy_image"))
-
-	// Don't get some k8s specific env
-	if !util.IsOpenShift {
-		defaultCheTLSSecretsCreationJobImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_tls_secrets_creation_job"))
-	}
-}
-
 func InitDefaultsFromFile(defaultsPath string) {
 	operatorDeployment := getDefaultsFromFile(defaultsPath)
 
@@ -130,6 +117,8 @@ func InitDefaultsFromFile(defaultsPath string) {
 	defaultPvcJobsImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_pvc_jobs"))
 	defaultPostgresImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_postgres"))
 	defaultKeycloakImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_keycloak"))
+	defaultSingleHostGatewayImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_single_host_gateway"))
+	defaultSingleHostGatewayConfigSidecarImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_single_host_gateway_config_sidecar"))
 	defaultCheWorkspacePluginBrokerMetadataImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_che_workspace_plugin_broker_metadata"))
 	defaultCheWorkspacePluginBrokerArtifactsImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_che_workspace_plugin_broker_artifacts"))
 	defaultCheServerSecureExposerJwtProxyImage = util.GetDeploymentEnv(operatorDeployment, util.GetArchitectureDependentEnv("RELATED_IMAGE_che_server_secure_exposer_jwt_proxy_image"))
@@ -161,15 +150,10 @@ func getDefaultsFromFile(defaultsPath string) *v1.Deployment {
 }
 
 func getDefaultFromEnv(envName string) string {
-
-	//old way
 	value := os.Getenv(envName)
-	// value := os.Getenv(util.GetArchitectureDependentEnv(envName)) // might be overkill to check ALL vars for an arch-specific version
 
 	if len(value) == 0 {
 		logrus.Fatalf("Failed to initialize default value: '%s'. Environment variable with default value was not found.", envName)
-		logrus.Fatalf("DEBUG s390x :: envName = '%s'", envName)
-		logrus.Fatalf("DEBUG s390x :: util.GetArchitectureDependentEnv(envName) = '%s'", util.GetArchitectureDependentEnv(envName))
 	}
 
 	return value
@@ -264,6 +248,14 @@ func DefaultCheServerSecureExposerJwtProxyImage(cr *orgv1.CheCluster) string {
 	return patchDefaultImageName(cr, defaultCheServerSecureExposerJwtProxyImage)
 }
 
+func DefaultSingleHostGatewayImage(cr *orgv1.CheCluster) string {
+	return patchDefaultImageName(cr, defaultSingleHostGatewayImage)
+}
+
+func DefaultSingleHostGatewayConfigSidecarImage(cr *orgv1.CheCluster) string {
+	return patchDefaultImageName(cr, defaultSingleHostGatewayConfigSidecarImage)
+}
+
 func DefaultPullPolicyFromDockerImage(dockerImage string) string {
 	tag := "latest"
 	parts := strings.Split(dockerImage, ":")
@@ -286,6 +278,14 @@ func GetCheMultiUser(cr *orgv1.CheCluster) string {
 	return DefaultCheMultiUser
 }
 
+func GetSingleHostExposureType(cr *orgv1.CheCluster) string {
+	if util.IsOpenShift {
+		return DefaultOpenShiftSingleHostExposureType
+	}
+
+	return util.GetValue(cr.Spec.K8s.SingleHostExposureType, DefaultKubernetesSingleHostExposureType)
+}
+
 func patchDefaultImageName(cr *orgv1.CheCluster, imageName string) string {
 	if !cr.IsAirGapMode() {
 		return imageName
@@ -342,3 +342,50 @@ func getOrganizationFromImage(image string) string {
 	}
 	return organization
 }
+
+func InitDefaultsFromEnv() {
+	defaultCheVersion = getDefaultFromEnv("CHE_VERSION")
+	defaultCheServerImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_server"))
+	defaultPluginRegistryImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_plugin_registry"))
+	defaultDevfileRegistryImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_devfile_registry"))
+	defaultPvcJobsImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_pvc_jobs"))
+	defaultPostgresImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_postgres"))
+	defaultKeycloakImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_keycloak"))
+	defaultSingleHostGatewayImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_single_host_gateway"))
+	defaultSingleHostGatewayConfigSidecarImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_single_host_gateway_config_sidecar"))
+
+	// CRW images for that are mentioned in the Che server che.properties
+	// For CRW these should be synced by hand with images stored in RH registries
+	// instead of being synced by script with the content of the upstream `che.properties` file
+	defaultCheWorkspacePluginBrokerMetadataImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_workspace_plugin_broker_metadata"))
+	defaultCheWorkspacePluginBrokerArtifactsImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_workspace_plugin_broker_artifacts"))
+	defaultCheServerSecureExposerJwtProxyImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_server_secure_exposer_jwt_proxy_image"))
+
+	// Don't get some k8s specific env
+	if !util.IsOpenShift {
+		defaultCheTLSSecretsCreationJobImage = getDefaultFromEnv(util.GetArchitectureDependentEnv("RELATED_IMAGE_che_tls_secrets_creation_job"))
+	}
+}
+
+func InitTestDefaultsFromDeployment(deploymentFile string) error {
+	operator := &appsv1.Deployment{}
+	data, err := ioutil.ReadFile(deploymentFile)
+	if err != nil {
+		return err
+	}
+
+	err = yaml.Unmarshal(data, operator)
+	if err != nil {
+		return err
+	}
+
+	for _, env := range operator.Spec.Template.Spec.Containers[0].Env {
+		err = os.Setenv(env.Name, env.Value)
+		if err != nil {
+			return err
+		}
+	}
+
+	InitDefaultsFromEnv()
+	return nil
+}
