@@ -12,6 +12,8 @@
 package deploy
 
 import (
+	"strings"
+
 	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
 )
 
@@ -19,3 +21,12 @@ func GetLabels(cr *orgv1.CheCluster, component string) (labels map[string]string
 	labels = map[string]string{"app": DefaultCheFlavor(cr), "component": component}
 	return labels
 }
+
+func MergeLabels(labels map[string]string, additionalLabels string) {
+	for _, l := range strings.Split(additionalLabels, ",") {
+		pair := strings.SplitN(l, "=", 2)
+		if len(pair) == 2 {
+			labels[pair[0]] = pair[1]
+		}
+	}
+}
