@@ -14,7 +14,6 @@ package deploy
 import (
 	"context"
 
-	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
 	"github.com/sirupsen/logrus"
 	rbac "k8s.io/api/rbac/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -25,26 +24,25 @@ import (
 )
 
 func SyncRoleBindingToCluster(
-	checluster *orgv1.CheCluster,
+	deployContext *DeployContext,
 	name string,
 	serviceAccountName string,
 	roleName string,
-	roleKind string,
-	clusterAPI ClusterAPI) (*rbac.RoleBinding, error) {
+	roleKind string) (*rbac.RoleBinding, error) {
 
-	specRB, err := getSpecRoleBinding(checluster, name, serviceAccountName, roleName, roleKind, clusterAPI)
+	specRB, err := getSpecRoleBinding(deployContext, name, serviceAccountName, roleName, roleKind)
 	if err != nil {
 		return nil, err
 	}
 
-	clusterRB, err := getClusterRoleBiding(specRB.Name, specRB.Namespace, clusterAPI.Client)
+	clusterRB, err := getClusterRoleBiding(specRB.Name, specRB.Namespace, deployContext.ClusterAPI.Client)
 	if err != nil {
 		return nil, err
 	}
 
 	if clusterRB == nil {
 		logrus.Infof("Creating a new object: %s, name %s", specRB.Kind, specRB.Name)
-		err := clusterAPI.Client.Create(context.TODO(), specRB)
+		err := deployContext.ClusterAPI.Client.Create(context.TODO(), specRB)
 		return nil, err
 	}
 
@@ -68,14 +66,13 @@ func getClusterRoleBiding(name string, namespace string, client runtimeClient.Cl
 }
 
 func getSpecRoleBinding(
-	checluster *orgv1.CheCluster,
+	deployContext *DeployContext,
 	name string,
 	serviceAccountName string,
 	roleName string,
-	roleKind string,
-	clusterAPI ClusterAPI) (*rbac.RoleBinding, error) {
+	roleKind string) (*rbac.RoleBinding, error) {
 
-	labels := GetLabels(checluster, DefaultCheFlavor(checluster))
+	labels := GetLabels(deployContext.CheCluster, DefaultCheFlavor(deployContext.CheCluster))
 	roleBinding := &rbac.RoleBinding{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "RoleBinding",
@@ -83,14 +80,14 @@ func getSpecRoleBinding(
 		},
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
-			Namespace: checluster.Namespace,
+			Namespace: deployContext.CheCluster.Namespace,
 			Labels:    labels,
 		},
 		Subjects: []rbac.Subject{
 			{
 				Kind:      rbac.ServiceAccountKind,
 				Name:      serviceAccountName,
-				Namespace: checluster.Namespace,
+				Namespace: deployContext.CheCluster.Namespace,
 			},
 		},
 		RoleRef: rbac.RoleRef{
@@ -100,7 +97,7 @@ func getSpecRoleBinding(
 		},
 	}
 
-	err := controllerutil.SetControllerReference(checluster, roleBinding, clusterAPI.Scheme)
+	err := controllerutil.SetControllerReference(deployContext.CheCluster, roleBinding, deployContext.ClusterAPI.Scheme)
 	if err != nil {
 		return nil, err
 	}
