@@ -27,6 +27,7 @@ import (
 	"strings"
 	"time"
 
+	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
 	"github.com/sirupsen/logrus"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -138,7 +139,6 @@ func GetServerResources() ([]*v1.APIResourceList, error) {
 }
 
 func GetValue(key string, defaultValue string) (value string) {
-
 	value = key
 	if len(key) < 1 {
 		value = defaultValue
@@ -146,8 +146,42 @@ func GetValue(key string, defaultValue string) (value string) {
 	return value
 }
 
-func IsTestMode() (isTesting bool) {
+func GetMapValue(value map[string]string, defaultValue map[string]string) map[string]string {
+	ret := value
+	if len(value) < 1 {
+		ret = defaultValue
+	}
 
+	return ret
+}
+
+func MergeMaps(first map[string]string, second map[string]string) map[string]string {
+	ret := make(map[string]string)
+	for k, v := range first {
+		ret[k] = v
+	}
+
+	for k, v := range second {
+		ret[k] = v
+	}
+
+	return ret
+}
+
+func GetServerExposureStrategy(c *orgv1.CheCluster, defaultValue string) string {
+	strategy := c.Spec.Server.ServerExposureStrategy
+	if IsOpenShift {
+		strategy = GetValue(strategy, defaultValue)
+	} else {
+		if strategy == "" {
+			strategy = GetValue(c.Spec.K8s.IngressStrategy, defaultValue)
+		}
+	}
+
+	return strategy
+}
+
+func IsTestMode() (isTesting bool) {
 	testMode := os.Getenv("MOCK_API")
 	if len(testMode) == 0 {
 		return false
@@ -156,11 +190,15 @@ func IsTestMode() (isTesting bool) {
 }
 
 func GetClusterPublicHostname(isOpenShift4 bool) (hostname string, err error) {
+	// Could be set for debug scripts.
+	CLUSTER_API_URL := os.Getenv("CLUSTER_API_URL")
+	if CLUSTER_API_URL != "" {
+		return CLUSTER_API_URL, nil
+	}
 	if isOpenShift4 {
 		return getClusterPublicHostnameForOpenshiftV4()
-	} else {
-		return getClusterPublicHostnameForOpenshiftV3()
 	}
+	return getClusterPublicHostnameForOpenshiftV3()
 }
 
 // getClusterPublicHostnameForOpenshiftV3 is a hacky way to get OpenShift API public DNS/IP
